@@ -2,7 +2,7 @@
  *
  *  littler - Provides hash-bang (#!) capability for R (www.r-project.org)
  *
- *  Copyright (C) 2006, 2007, 2008 Jeffrey Horner and Dirk Eddelbuettel
+ *  Copyright (C) 2006 - 2009  Jeffrey Horner and Dirk Eddelbuettel
  *
  *  littler is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: littler.c 145 2008-08-19 16:47:23Z jeffrey.horner $
+ *  $Id: littler.c 157 2009-01-14 02:08:23Z dirk.eddelbuettel $
  */
 
 #include <getopt.h>
@@ -42,20 +42,10 @@
 #define R_INTERFACE_PTRS
 #include <Rinterface.h>
 #include <R_ext/Parse.h>
+#include <R_ext/RStartup.h>
 
 int verbose=0;
 
-#if R_VERSION < R_Version(2,5,0)
-extern int R_Visible; /* We're cheating here, as this is undocumented and not in the
-                       * R embedded interface.
-					   * 
-                       * This variable controls when an expression is printed
-                       * via PrintValue(). 
-					   *
-					   * Looks like this will go away in R 2.5.x
-                       */
-#endif
-
 /* these two are being filled by autoconf and friends via config.h */
 /* VERSION */
 const char *programName = PACKAGE;
@@ -294,11 +284,7 @@ int parse_eval(membuf_t *pmb, char *line, int lineno){
 	SET_STRING_ELT(cmdSexp, 0, mkChar((char*)mb->buf));
 
 	/* R_ParseVector gets a new argument in R 2.5.x */
-	#if R_VERSION >= R_Version(2,5,0)
-	cmdexpr = PROTECT(R_ParseVector(cmdSexp, -1, &status,R_NilValue));
-	#else
-	cmdexpr = PROTECT(R_ParseVector(cmdSexp, -1, &status));
-	#endif
+	cmdexpr = PROTECT(R_ParseVector(cmdSexp, -1, &status, R_NilValue));
 
 	switch (status){
 		case PARSE_OK:
@@ -307,21 +293,15 @@ int parse_eval(membuf_t *pmb, char *line, int lineno){
 				ans = R_tryEval(VECTOR_ELT(cmdexpr, i),NULL,&errorOccurred);
 				if (errorOccurred) return 1;
 
-				#if R_VERSION < R_Version(2,5,0)
-				/* We won't be able to test for R_Visible after R 2.5.x.
-				 * need to figure out why...
-				 */
-				if (verbose && R_Visible){
-				#else 
 				if (verbose){
-				#endif
 					PrintValue(ans);
 				}
 			}
 			mb = *pmb = rewind_membuf(pmb);
 		break;
 		case PARSE_INCOMPLETE:
-			/* need to read another line */
+			fprintf(stderr, "%s: Incomplete Line! Need more code!\n", programName, status);
+			return 1;
 		break;
 		case PARSE_NULL:
 			fprintf(stderr, "%s: ParseStatus is null (%d)\n", programName, status);
@@ -332,7 +312,7 @@ int parse_eval(membuf_t *pmb, char *line, int lineno){
 			return 1;
 		break;
 		case PARSE_EOF:
-			fprintf(stderr, "%s: ParseStatus is eof (%d)\n", programName, status);
+			fprintf(stderr, "%s: EOF reached (%d)\n", programName, status);
 		break;
 		default:
 			fprintf(stderr, "%s: ParseStatus is not documented %d\n", programName, status);
@@ -344,11 +324,14 @@ int parse_eval(membuf_t *pmb, char *line, int lineno){
 }
 
 extern char *R_TempDir;
+int perSessionTempDir = FALSE;	/* by default, r differs from R and defaults to /tmp unless env.vars set, or flag chosen */
 
 void littler_InitTempDir()
 {
 	char *tmp;
 
+	if (perSessionTempDir) return; /* use a per-session temporary directory by following R */
+
 	tmp = getenv("TMPDIR");
 	if (tmp == NULL) {
 		tmp = getenv("TMP");
@@ -367,8 +350,9 @@ void littler_InitTempDir()
 	}
 }
 
-/* Called via q() or if an error condition occurs and getOption("error") == NULL */
+/* littler exit */
 void littler_CleanUp(SA_TYPE saveact, int status, int runLast){
+	if (perSessionTempDir) R_CleanTempDir();
 	exit(status);
 }
 
@@ -384,6 +368,8 @@ void showHelpAndExit() {
 	       "      --usage          Give a short usage message\n"
 	       "  -V, --version        Show the version number\n"
 	       "  -v, --vanilla        Pass the '--vanilla' option to R\n"  
+	       "  -t, --rtemp          Use per-session temporary directory as R does\n"
+	       "  -q, --quick          Skip autoload / delayed assign of default libraries\n"
 	       "  -p, --verbose        Print the value of expressions to the console\n"  
 	       "  -l, --packages list  Load the R packages from the comma-separated 'list'\n"	
 	       "  -e, --eval  expr     Let R evaluate 'expr'\n"
@@ -411,7 +397,7 @@ void showVersionAndExit() {
 					R_SVN_REVISION);
 		}
 	}
-  	printf("\n\nCopyright (C) 2006, 2007, 2008 Jeffrey Horner and Dirk Eddelbuettel\n"
+  	printf("\n\nCopyright (C) 2006 - 2009 Jeffrey Horner and Dirk Eddelbuettel\n"
 	       "\n"
 	       "%s is free software and comes with ABSOLUTELY NO WARRANTY.\n"
 	       "You are welcome to redistribute it under the terms of the\n"
@@ -469,7 +455,7 @@ int main(int argc, char **argv){
 	char *R_argv[] = {(char*)programName, "--gui=none", "--no-restore", "--no-save", "--no-readline", "--silent", "", ""};
 	char *R_argv_opt[] = {"--vanilla", "--slave"};
 	int R_argc = (sizeof(R_argv) - sizeof(R_argv_opt) ) / sizeof(R_argv[0]);
-	int i, nargv, c, optpos=0, vanilla=0;
+	int i, nargv, c, optpos=0, vanilla=0, quick=0;
 	char *evalstr = NULL;
 	char *libstr = NULL;
 	SEXP s_argv;
@@ -482,9 +468,11 @@ int main(int argc, char **argv){
 		{"eval",    required_argument, NULL, 'e'},
 		{"packages",required_argument, NULL, 'l'},
 		{"verbose", no_argument,       NULL, 'p'},
+		{"rtemp",   no_argument,       NULL, 't'},
+		{"quick",   no_argument,       NULL, 'q'},
 		{0, 0, 0, 0}
 	};
-	while ((c = getopt_long(argc, argv, "+hVve:npl:", optargs, &optpos)) != -1) {
+	while ((c = getopt_long(argc, argv, "+hVve:npl:tq", optargs, &optpos)) != -1) {
 		switch (c) {	
 			case 0:					/* numeric 0 is code for a long option */
 				/* printf ("Got option %s %d", optargs[optpos].name, optpos);*/
@@ -521,6 +509,12 @@ int main(int argc, char **argv){
 			case 'V':
 				showVersionAndExit();
 				break;  			/* never reached */
+			case 't':
+				perSessionTempDir=TRUE;
+				break;
+			case 'q':
+				quick=1;
+				break;
 			default:
 				printf("Unknown option '%c'. Try `%s --help' for help\n",(char)c, programName);
 				exit(-1);
@@ -557,7 +551,6 @@ int main(int argc, char **argv){
 	}
 
 	/* Setenv R_* env vars: insert or replace into environment.  */
-
 	for (i = 0; R_VARS[i] != NULL; i+= 2){
 		if (setenv(R_VARS[i],R_VARS[i+1],1) != 0){
 			perror("ERROR: couldn't set/replace an R environment variable");
@@ -589,8 +582,10 @@ int main(int argc, char **argv){
 
 	ptr_R_CleanUp = littler_CleanUp;
 
-	/* Force all default package to be dynamically required */
-	autoloads();
+	/* Force all default package to be dynamically required, unless chosen not to */
+	if (quick != 1) {
+		autoloads();
+	}
 
 	/* Place any argv arguments into argv vector in Global Environment */
 	/* if we have an evalstr supplied from -e|--eval, correct for it */
@@ -654,5 +649,6 @@ int main(int argc, char **argv){
 		destroy_membuf(lb);
 		destroy_membuf(pb);
 	}
-	exit(exit_val);
+	littler_CleanUp(SA_NOSAVE, exit_val,0);
+	/* not reached */
 }
